package org.apache.bcel.classfile;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;
import org.apache.bcel.Constants;

public final class CodeException implements Cloneable, Node, Constants
{
    static final CodeException[] EMPTY_CODE_EXCEPTION_ARRAY;
    private int startPc;
    private int endPc;
    private int handlerPc;
    private int catchType;
    
    public CodeException(final CodeException c) {
        this(c.getStartPC(), c.getEndPC(), c.getHandlerPC(), c.getCatchType());
    }
    
    CodeException(final DataInput file) throws IOException {
        this(file.readUnsignedShort(), file.readUnsignedShort(), file.readUnsignedShort(), file.readUnsignedShort());
    }
    
    public CodeException(final int startPc, final int endPc, final int handlerPc, final int catchType) {
        this.startPc = startPc;
        this.endPc = endPc;
        this.handlerPc = handlerPc;
        this.catchType = catchType;
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitCodeException(this);
    }
    
    public CodeException copy() {
        try {
            return (CodeException)this.clone();
        }
        catch (final CloneNotSupportedException ex) {
            return null;
        }
    }
    
    public void dump(final DataOutputStream file) throws IOException {
        file.writeShort(this.startPc);
        file.writeShort(this.endPc);
        file.writeShort(this.handlerPc);
        file.writeShort(this.catchType);
    }
    
    public int getCatchType() {
        return this.catchType;
    }
    
    public int getEndPC() {
        return this.endPc;
    }
    
    public int getHandlerPC() {
        return this.handlerPc;
    }
    
    public int getStartPC() {
        return this.startPc;
    }
    
    public void setCatchType(final int catchType) {
        this.catchType = catchType;
    }
    
    public void setEndPC(final int endPc) {
        this.endPc = endPc;
    }
    
    public void setHandlerPC(final int handlerPc) {
        this.handlerPc = handlerPc;
    }
    
    public void setStartPC(final int startPc) {
        this.startPc = startPc;
    }
    
    @Override
    public String toString() {
        return "CodeException(startPc = " + this.startPc + ", endPc = " + this.endPc + ", handlerPc = " + this.handlerPc + ", catchType = " + this.catchType + ")";
    }
    
    public String toString(final ConstantPool cp) {
        return this.toString(cp, true);
    }
    
    public String toString(final ConstantPool cp, final boolean verbose) {
        String str;
        if (this.catchType == 0) {
            str = "<Any exception>(0)";
        }
        else {
            str = Utility.compactClassName(cp.getConstantString(this.catchType, (byte)7), false) + (verbose ? ("(" + this.catchType + ")") : "");
        }
        return this.startPc + "\t" + this.endPc + "\t" + this.handlerPc + "\t" + str;
    }
    
    static {
        EMPTY_CODE_EXCEPTION_ARRAY = new CodeException[0];
    }
}
